//
//  WBOverviewPanel.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 6/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "WBOverviewPanel.h"
#import "MTogglePane.h"
#import "WBOverviewListController.h"

@interface WBOverviewBackgroundView : NSView
{
  NSImage *bgImage;
  NSImage *shadowImage;
  float usedHeight;
}
@end

static void DrawTiledImage(NSImage *image, NSRect rect, BOOL composite)
{
  NSSize imageSize= [image size];
  NSRect imageRect;
  
  imageRect.origin= NSMakePoint(0, 0);
  imageRect.size= imageSize;
  for (CGFloat y= 0; y < rect.size.height; y+= imageSize.height)
  {
    for (CGFloat x= 0; x < rect.size.width; x+= imageSize.width)
    {
      [image drawAtPoint:NSMakePoint(rect.origin.x+x, rect.origin.y+y) 
                fromRect:imageRect
               operation:composite ? NSCompositeSourceOver : NSCompositeCopy 
                fraction:1.0];
    }
  }
}

@implementation WBOverviewBackgroundView
- (id)initWithFrame:(NSRect)rect
{
  if ((self= [super initWithFrame:rect]) != nil)
  {
    bgImage= [[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForResource:@"background" ofType:@"png" inDirectory:@"images"]];
    shadowImage= [[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForResource:@"background_top_shadow" ofType:@"png" inDirectory:@"images"]];
    [shadowImage setFlipped:YES];
  }
  return self;
}

- (BOOL)isFlipped
{
  return YES;
}

- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [bgImage release];
  [shadowImage release];
  [super dealloc];
}

- (void)drawRect:(NSRect)rect
{  
  rect= [self frame];
  if (usedHeight < NSHeight(rect))
  {
    rect.size.height-= usedHeight;
    rect.origin.y= 0;
    
    DrawTiledImage(bgImage, rect, NO);
    rect.origin.y= usedHeight;
    rect.size.height= [shadowImage size].height;
    DrawTiledImage(shadowImage, rect, YES);
  }
}


- (void)relayout
{
  float availableHeight;
  float y;
  float oldUsedHeight= usedHeight;
  
  usedHeight= 0;
  for (NSView *view in [self subviews])
    usedHeight+= NSHeight([view frame]);
  
  if (oldUsedHeight == usedHeight)
    return;
  
  availableHeight= NSHeight([[self superview] frame]);
  NSLog(@"mine=%f  par=%f   tot=%f", 
        NSHeight([self frame]), availableHeight, usedHeight);
  
  [self setFrameSize:NSMakeSize(NSWidth([self frame]), usedHeight < availableHeight ? availableHeight : usedHeight)];

  y= 0;
  for (NSView *view in [self subviews])
  {
    [view setFrameOrigin:NSMakePoint(0, y)];
    y+= NSHeight([view frame]);
  }
  
  [self setNeedsDisplay:YES];
}



- (void)childFrameChanged:(NSNotification*)notif
{
  [self relayout];
}


- (void)didAddSubview:(NSView*)subview
{
  [subview setPostsFrameChangedNotifications:YES];
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(childFrameChanged:)
                                               name:NSViewFrameDidChangeNotification
                                             object:subview];
  
  [super didAddSubview:subview];
}

@end





@implementation WBOverviewPanel

- (id)initWithBE:(wb::OverviewBE*)overview
{
  if ((self= [super init]) != nil)
  {
    _overview= overview;

    _scrollView= [[NSScrollView alloc] initWithFrame:NSMakeRect(0,0, 300, 300)];
    [_scrollView setHasVerticalScroller:YES];
    [_scrollView setHasHorizontalScroller:NO];
    
    _contentView= [[WBOverviewBackgroundView alloc] initWithFrame:NSMakeRect(0, 0, 
                                                           NSWidth([_scrollView frame])-[NSScroller scrollerWidth],
                                                           NSHeight([_scrollView frame]))];
    [_contentView setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
    [_scrollView setDocumentView:_contentView];
  }
  return self;
}


- (void)dealloc
{
  [_scrollView release];
  [super dealloc];
}


- (NSView*)topView
{
  return _scrollView;
}


- (NSString*)title
{
  return @"MySQL Model";
}


- (NSString*)identifier
{
  return @"overview";
}


- (bec::UIForm*)formBE
{
  return _overview;
}


- (void)setup
{
  [self buildMainSections];
}


- (void)refresh
{
}


static int first_child_type(wb::OverviewBE *overview, const bec::NodeId &node)
{
  if (overview->count_children(node) > 0)
  {
    int type= -1;
    overview->get_field(overview->get_child(node, 0), wb::OverviewBE::NodeType, type);
    return type;
  }
  return -1;
}


- (NSView*)buildIconView:(const bec::NodeId&)node
{
  NSMutableArray *objects= [NSMutableArray array];
  NSDictionary *nameTable= [NSDictionary dictionaryWithObject:objects forKey:NSNibTopLevelObjects];
  
  [NSBundle loadNibFile:[[NSBundle mainBundle] pathForResource:@"IconCollectionView"
                                                        ofType:@"nib"]
      externalNameTable:nameTable withZone:nil];

  NSCollectionView *collection= nil;
  WBOverviewListController *controller= nil;
  
  for (id object in objects)
  {
    if ([object isKindOfClass:[NSCollectionView class]])
      collection= (NSCollectionView*)object;

    if ([object isKindOfClass:[WBOverviewListController class]])
      controller= (WBOverviewListController*)object;
  }
    
  [controller setOverviewBE:_overview];
  
  [controller fillFromChildrenOf:node
                        overview:_overview
                        iconSize:bec::Icon32];
    
  return collection;
}


- (NSView*)buildDivision:(const bec::NodeId&)node
{
  if (_overview->count_children(node) > 0)
  {
    int type;
    // Get the type of the item we are adding. It could be division (EER Diagrams, Physical Schemata)
    // or group (database, ...)
    // or TODO: add comments here for the rest of the stuff
    // type is modified inside the get_field
    _overview->get_field(_overview->get_child(node, 0), wb::OverviewBE::NodeType, type);

    if (type == wb::OverviewBE::OGroup)
    {
      
    }
    else if (type == wb::OverviewBE::OItem)
    {
      return [self buildIconView:node];
    }
    else
    {
    }
  }
  return nil;
}


- (void)buildMainSections
{
  bec::NodeId root(1), node;
  CGFloat width= NSWidth([_contentView frame]);

  _overview->refresh();
  for (int i= 0; i < _overview->count_children(root); i++)
  {
    std::string label;
    wb::OverviewBE::OverviewNodeType nodeType;
    int expanded;

    node= _overview->get_child(root, i);
    
    _overview->get_field(node, wb::OverviewBE::Label, label);
    _overview->get_field(node, wb::OverviewBE::NodeType, (int&)nodeType);
    _overview->get_field(node, wb::OverviewBE::Expanded, expanded);
    
    if (nodeType == wb::OverviewBE::ODivision)
    {
      MTogglePane *pane= [[MTogglePane alloc] initWithFrame:NSMakeRect(0, 0, width, 100)];
      [pane setLabel:[NSString stringWithUTF8String:label.c_str()]];
      [pane setAutoresizingMask:NSViewWidthSizable|NSViewMaxYMargin];
  
      NSView *view= [self buildDivision:node];      

      [pane setContentView:view];
    
      [pane setExpanded:expanded!=0];

      [_contentView addSubview:pane];
    }
    else
    {
      NSLog(@"ERROR: unexpected node type in Overview");
    }
  }
  [_contentView relayout];
}

@end
