#ifndef _TUT_HELPERS_H_
#define _TUT_HELPERS_H_

#ifndef __WIN__
#undef max
#endif

#include <stack>

#ifndef _WIN32
#include <iostream>
#include <sys/time.h>
#endif

#ifdef _WIN32
#include <windows.h>
#endif

#include "tut.h"
#include <list>
/*
namespace tut
{
  template <class Data,int MaxTestsInGroup = 50>
  class test_group_with_desc : public test_group<Data, MaxTestsInGroup>
  {
    const char* desc_;

  public:

    test_group_with_desc(const char* name, const char *desc)
    : test_group(name), desc_(desc)
    {}
  }
}
*/

namespace tut
{
  class test_time_point
  {
#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
    long ticks_;
    test_time_point(DWORD t)
    : ticks_(t)
    {}
  public:
    test_time_point()
    : ticks_(GetTickCount())
    {}

    long get_ticks() const { return ticks_; }

    friend test_time_point operator - (const test_time_point& t1,
                                       const test_time_point& t2);

    friend std::ostream& operator << (std::ostream& os, const test_time_point& ttp);
#else
  public:
    long ticks_;
    test_time_point(long t)
    : ticks_(t)
    {}
    test_time_point()
    {
      struct timeval tv;
      gettimeofday(&tv, NULL);
      ticks_= tv.tv_sec * 1000 + tv.tv_usec / 1000;
    }
#endif

  };

  inline test_time_point operator - (const test_time_point& t1,
                              const test_time_point& t2)
  {
    return test_time_point(t1.ticks_ - t2.ticks_);
  }

  inline std::ostream& operator << (std::ostream& os, const test_time_point& ttp)
  {
    long sec= ttp.ticks_ / 1000;
    long ms=  ttp.ticks_ % 1000;

    os << sec << "." << ms << " sec";

    return os;
  }

  /*
    debug_stringbuf is used by debug_ostream to
    perform output to an OS debugging output facility
  */

  class debug_stringbuf :
    public std::basic_stringbuf<char, std::char_traits<char> >
  {
  public:

    virtual int sync()
    {
#ifdef _WIN32
      OutputDebugStringA(str().c_str());
#else
      std::cerr << str();  // todo: change this?
#endif
      str("");  // set empty string - clean the buffer
      return 0;
    }
  };

  class debug_ostream : public std::ostream
  {
  public:
    debug_ostream()
    : std::ostream(new debug_stringbuf())
    {}

    ~debug_ostream()
    {
      delete rdbuf();
    }
  };

  template<typename T = int>
  struct leveled_counter
  {
    leveled_counter() { push_level(); }
    ~leveled_counter() {}
  
    void push_level() { _stack.push(0);  }
    void pop_level() { T top= _stack.top();  _stack.pop(); _stack.top()+= top; };
  
    T count() const { return _stack.top(); }
    void incr(T i= 1) { _stack.top() += i; }

    int level_count() const { return (int)_stack.size() - 1; }
  
  private:
    std::stack<T> _stack;
  };

  class mysql_reporter : public tut::callback
  {
    std::ostream *os;

    leveled_counter<int> _failed;
    leveled_counter<int> _passed;

    test_time_point start_time;

    std::list<std::string> failed_modules;

    bool _reportpass;
  public:

    mysql_reporter(bool debugout= false, bool reportpass= true)
    : os(NULL), _reportpass(reportpass)
    {
      if(debugout)
      {
        os= new debug_ostream;
      }
      else
      {
        os= &std::cout;
      }
      init();
    }

    ~mysql_reporter()
    {
      if(os != &std::cout)
        delete os;
    }

    std::string print_offset()
    {
      int tab_count = _passed.level_count() - 1;
      if (tab_count < 0) 
        tab_count = 0;
      return std::string(tab_count, '\t');
    }

    void init()
    {
      start_time= test_time_point();

//      if(group_base::get_disabled_groups_count() > 0)
//        *os << "*** Disabled modules: " << group_base::get_disabled_groups_count() << std::endl;
    }

    void print_summary()
    {
      *os << std::endl << "Run complete. [";
      if(_reportpass)
        *os << "passed:" << _passed.count();

      if(_failed.count() > 0)
      {
        *os <<  " failed:" << _failed.count();
        *os <<  " failed modules:" << failed_modules.size();
        for (std::list<std::string>::iterator iter= failed_modules.begin(); iter != failed_modules.end(); ++iter)
          *os << " - " << *iter;
      }
      *os << "]"
        << std::endl << "Duration:" << (test_time_point() - start_time)
        << std::endl 
        << std::endl;
    }

    void group_started(const std::string& name)
    {
      _failed.push_level();
      _passed.push_level();
      *os << std::endl << print_offset() << "Module " << name << std::endl/* << std::endl*/;
    }

    void test_completed(const test_result& tr)
    {
      if((tr.result == test_result::ok))
      {
        _passed.incr();
        if(_reportpass)
          *os << print_offset() << tr.test << ": passed" << std::endl;
      }
      else
      {
        _failed.incr();
#ifdef _WIN32
        SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE),
          FOREGROUND_RED|FOREGROUND_INTENSITY);
#endif
        *os << print_offset() << tr.test << ": failed (" << tr.message << ")" << std::endl;
#ifdef _WIN32
        SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE),
          FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
#endif
      }
    }

    void group_completed(const std::string& name)
    {
      *os << print_offset() << /*std::endl <<*/ "Module "<< name << " [";
      if(_reportpass)
        *os << "passed:" << _passed.count();
      if(_failed.count() > 0)
        *os << " failed:" << _failed.count();
      *os << "]" << std::endl;

      if (_failed.count())
        failed_modules.push_back(name);

      _failed.pop_level();
      _passed.pop_level();
    }

    bool succeeded() const { return !_failed.count(); }
  };
};

#endif // _TUT_HELPERS_H_
